unit unitMain;
{

    BOINCprog - show status and progress of current WU on single-CPU system
    Copyright (C) 2004, 2005  Trane Francks

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Comments, suggestions and bug reports may be sent electronically to:

    Trane Francks <trane@gol.com>

    Snail-mail should be addressed to:

    Trane Francks
    Sento Hills Tsurukawa #305
    1856-4 Kanai-cho, Machida-shi
    Tokyo-to 195-0071 JAPAN

}



interface

uses
  SysUtils, Types, Classes, Variants, QTypes, QGraphics, QControls, QForms,
  QDialogs, QStdCtrls, QExtCtrls, QComCtrls, DateUtils, QMenus, QStyle,
  QTrayIcon;

type
  TfrmMain = class(TForm)
    lblCurrentWorkUnit: TLabel;
    ProgressBar1: TProgressBar;
    Timer1: TTimer;
    lblPercent: TLabel;
    lblCPU: TLabel;
    lblProject: TLabel;
    lblReportDeadline: TLabel;
    lblReportDeadlineValue: TLabel;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Exit2: TMenuItem;
    TrayIcon1: TTrayIcon;
    PopupMenu1: TPopupMenu;
    Restore1: TMenuItem;
    Exit3: TMenuItem;
    Hide1: TMenuItem;
    Hide2: TMenuItem;
    About1: TMenuItem;
    Timer2: TTimer;
    procedure Timer1Timer(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure Exit2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Restore1Click(Sender: TObject);
    procedure Exit3Click(Sender: TObject);
    procedure Hide1Click(Sender: TObject);
    procedure Hide2Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Timer2Timer(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
  public
    { Public declarations }
  end;


const
  strActiveTask = '<active_task>';
  strResultName = '<result_name>';
  strFractionDone = '<fraction_done>';
  strCurrentCpuTime = '<current_cpu_time>';
  strProjectName = '<project_master_url>';
  strSchedulerState = '<scheduler_state>';


var
  txtFilename     : string;
  frmMain         : TfrmMain;
  workFile        : text;
  currentString   : string;
  scheduler_state : integer;
  project_name    : string;
  counter         : integer;
  use_kde_panel   : boolean;

implementation

uses unitAbout, unitPanel, Splash;

{$R *.xfm}

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  if (GetEnvironmentVariable('USE_KDE_PANEL') = '1') then
  begin
    use_kde_panel := true;
    frmMain.Hide;
    TrayIcon1.Show;
  end
  else
  begin
    use_kde_panel := false;
    counter := 0;
    Timer2.Enabled := true;
  end;
end;

procedure TfrmMain.Timer1Timer(Sender: TObject);
var
  intPos          : string[2];
  code            : integer;
  progress        : real;
  foundActiveTask : boolean;
  cpuTime         : real;
  tmpString       : string;
  foundResult     : boolean;
  unixTimeString  : string;
  unixTime        : int64;
  boincDir        : string;

begin
  foundResult := false;
  scheduler_state := 0;
  foundActiveTask := false;
  boincDir := paramstr(1);
  txtFilename := boincDir + '/client_state.xml';
  If FileExists(txtFilename) then
  begin
    assignfile(workFile, txtFilename);
    frmMain.Caption := boincDir;
    TrayIcon1.ToolTip := boincDir;
  end
  else
  begin { no file }
    lblCurrentWorkUnit.Caption := 'client_state.xml not found';
    exit;
  end; { no file }
  filemode := 0;
  reset(workFile);

    { First, determine whether we even have an active task to report }
    while (pos(strActiveTask, currentString) = 0) and not eof(workFile) do
    begin { find task }
      readln(workFile, currentString);
      if (pos(strActiveTask, currentString) <> 0) then
        foundActiveTask := true;
    end; { find task }
    if (foundActiveTask <> true) then
    begin { break }
      lblProject.Caption := 'No active project';
      lblCurrentWorkUnit.Caption := 'No work in progress';
      ProgressBar1.Position := 0;
      lblPercent.Caption := '';
      lblCPU.Caption := '';
      { 2006-08-15: Properly clear deadline when we run out of work. }
      lblReportDeadlineValue.Caption := '';
      { /2006-08-15 processing }
      closefile(workFile);
      exit;
    end; { break }

  { If we got this far, we have workunits to crunch. Find the one running. }
  { find scheduler_state = 2  ('2' means that it's running, not preempted. }
  while (pos('<scheduler_state>2', currentString) = 0) and not eof(workFile) do
  begin { find state }
    readln(workFile, currentString);
    if (pos(strProjectName, currentString) <> 0) then
    begin { get project name }
      delete(currentString, 1, pos(strProjectName, currentString) + 19);
      delete(currentString, length(currentString) - 20, 21);
      project_name := currentString;
    end; { get project name }
  end; { find state }
  closefile(workFile);

   { reset the client_state file and find the active_task section }
  foundActiveTask := false;
  filemode := 0;
  reset(workFile);
    while (pos(strActiveTask, currentString) = 0) and not eof(workFile) do
    begin { find task }
      readln(workFile, currentString);
      if (pos(strActiveTask, currentString) <> 0) then
        foundActiveTask := true;
    end; { find task }

  { Now, find our currently running project URL }
  while (pos(project_name, currentString) = 0) and not eof(workFile) do
    readln(workFile, currentString);
  delete(currentString, 1, pos(strProjectName, currentString) + 19);
  delete(currentString, length(currentString) - 20, 21);
  lblProject.Caption := currentString;


  while (pos(strResultName, currentString) = 0) and not eof(workFile) do
    readln(workFile, currentString);
  delete(currentString, 1, pos(strResultName, currentString) + 12);
  delete(currentString, length(currentString) - 13, 14);
  lblCurrentWorkUnit.Caption := currentString;

    while (pos(strFractionDone, currentString) = 0) and not eof(workFile) do
      readln(workFile, currentString);
    delete(currentString, 1, pos(strFractionDone, currentString) + 14);
    delete(currentString, length(currentString) - 15, 16);
    val(currentString, progress, code);
    if (pos('1', currentString) = 1) then
    begin { graceful exit }
      ProgressBar1.Position := 100;
      lblPercent.Caption := '100%';
    end { graceful exit }
    else
    begin { process progress value }
      ProgressBar1.Caption := currentString;
      delete(currentString, 1, 2);
      intPos := currentString[1] + currentString[2];
      ProgressBar1.Position := StrToInt(intPos);
      progress := progress * 100;
      lblPercent.Caption := FormatFloat('0.000', progress) + '%';
    end; { process progress value }

    while (pos(strCurrentCpuTime, currentString) = 0) and not eof(workFile) do
      readln(workFile, currentString);
    delete(currentString, 1, pos(strCurrentCpuTime, currentString) + 17);
    delete(currentString, length(currentString) - 18, 19);
    val(currentString, cpuTime, code);
    tmpString := FormatFloat('0.000', cpuTime /60 /60);
    lblCPU.Caption := tmpString + ' hours';


    closefile(workFile);
    filemode := 0;
    reset(workFile);
    currentString := '';
    while not foundResult and not eof(workFile) do
    begin { find <result> tag for current WU }
      while (pos('<result>', currentString) =0) and not eof(workFile) do
        readln(workFile, currentString);
      readln(workFile, currentString);
      if (pos(lblCurrentWorkUnit.Caption, currentString) <> 0) then
      begin { found result name }
        foundResult := true;
        while (pos('<report_deadline>', currentString) = 0) and not
               eof(workFile) do
          readln(workFile, currentString);
        delete(currentString, 1, pos('<report_deadline>', currentString) +
               16);
        delete(currentString, length(currentstring) - 17, 18);
        unixTimeString := currentString;
        { 2005-06-04: Add processing to cope with Real deadline instead of }
        { Integer data type found in 4.0 - 4.19 clients                    }
        if (pos('.', unixTimeString) <> 0) then
          delete(unixTimeString, pos('.', unixTimeString),
                length(unixTimeString) + 1 - pos('.', unixTimeSTring));
        { /2005-06-04 processing                                            }
      end; { found result name }
    end; { find <result> tag for current WU }
    { convert UNIX timestamp to human-readable date/time }
    unixTime := StrToInt64(unixTimeString);
    lblReportDeadlineValue.Caption := DateTimeToStr(UnixToDateTime(unixTime)) +
                                        ' UT';
    closeFile(workFile);


end; { procedure TfrmMain.Timer1Timer(Sender: TObject) }

procedure TfrmMain.Exit1Click(Sender: TObject);
begin
  AboutBox.Show
end;

procedure TfrmMain.Exit2Click(Sender: TObject);
begin
  frmSplash.Close;
end;

procedure TfrmMain.Restore1Click(Sender: TObject);
begin
  frmMain.Show;
  Restore1.Enabled := false;
  Hide2.Enabled := true;
end;

procedure TfrmMain.Exit3Click(Sender: TObject);
begin
  frmSplash.Close;
end;

procedure TfrmMain.Hide1Click(Sender: TObject);
begin
  self.Visible := false;
  self.Hide;
  Hide2.Enabled := false;
  Restore1.Enabled := true;
end;

procedure TfrmMain.Hide2Click(Sender: TObject);
begin
  frmMain.Hide;
  Restore1.Enabled := True;
  Hide2.Enabled := False;
end;

procedure TfrmMain.About1Click(Sender: TObject);
begin
  AboutBox.Show;
  AboutBox.SetFocus;
end;

procedure TfrmMain.Timer2Timer(Sender: TObject);
begin
  inc(counter);
  if (counter > 2) then
  begin
    frmMain.Show;
    Hide1.Enabled := false;
    Timer2.Enabled := false;
  end;
end;

{ 2005-12-27: Add logic to properly terminate program when user closes }
{ main form and not starting up as KDE applet.                         }
procedure TfrmMain.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if not use_kde_panel then
    frmSplash.Close;
end;
{ /2005-12-27 processing }

end.

